﻿Option Explicit On

'  ヒュッケル分子軌道法計算のプログラム　=== コンソール版　===

'                       Written by Atsushi Hata and Hiroshi Kihara

'  奇数電子系およびHOMOの二重縮退に対応しています。

Module Module1

    Dim G(20, 20) As Double          ' 永年行列
    Dim V(20, 20) As Double          ' 固有ベクトル
    Dim P(20, 20) As Double          ' 結合次数
    Dim nAtom As Integer             ' 化合物名
    Dim nBond As Integer             ' 原子数
    Dim nPiElectron As Integer       ' π電子数
    Dim Title As String

    Sub Main()

        Dim i As Integer
        Dim j As Integer
        Dim k As Integer
        Dim nHetero As Integer
        Dim Hij As Double
        Dim prompt As String
        Dim buf As String
        Dim Array() As String

        Console.WriteLine("　ヒュッケル分子軌道法の計算プログラム")
        Console.Write("化合物名: ")
        Title = Console.ReadLine()
        Console.Write("原子数: ")
        nAtom = Console.ReadLine()

        If nAtom < 2 Then
            Console.WriteLine("原子数は２以上を入力してください。")
            Console.WriteLine("何かキーを押してください。")
            Console.ReadKey()
            Exit Sub
        End If
        If nAtom > 20 Then
            Console.WriteLine("原子数の上限は20です。")
            Console.WriteLine("何かキーを押してください。")
            Console.ReadKey()
            Exit Sub
        End If

        Console.Write("結合数: ")
        nBond = Console.ReadLine()
        Console.Write("電子数: ")
        nPiElectron = Console.ReadLine()
        Console.Write("ヘテロ・パラメータ数: ")
        nHetero = Console.ReadLine()
        Console.WriteLine()
        Console.WriteLine()

        Console.WriteLine("結合している原子の組を ，で区切って入力してください")
        Console.WriteLine()

        For k = 1 To nBond
            prompt = "[" + Trim(Str(k)) + "]: "
            Console.Write(prompt)
            buf = Console.ReadLine()
            Array = Split(buf, ",")
            i = Val(Array(0))
            j = Val(Array(1))
            G(i, j) = 1.0#         '永年行列の要素
            G(j, i) = 1.0#
        Next k

        If nHetero > 0 Then
            Console.WriteLine()
            Console.WriteLine()
            Console.WriteLine("ヘテロ・パラメータの行列の位置と値を ，で区切って入力してください")
            Console.WriteLine()

            For k = 1 To nHetero
                prompt = "[" + Trim(Str(k)) + "]: "
                Console.Write(prompt)
                buf = Console.ReadLine()
                Array = Split(buf, ",")
                i = Val(Array(0))
                j = Val(Array(1))
                Hij = Val(Array(2))
                G(i, j) = Hij         '永年行列の要素
                G(j, i) = Hij
            Next k

        End If

        Call DIAG()               'ヤコビ法による対角化

        Call INCRE()              ' 固有ベクトルを固有値の順に並べかえる

        Call PMAT()               ' 結合次数の計算

        Call POUT()               ' 結果の表示

        Console.WriteLine()
        Console.WriteLine("何かキーを押してください。")
        Console.ReadKey()

    End Sub

    Sub DIAG()

        Dim DL As Double
        Dim D As Double
        Dim SM As Double
        Dim CS As Double
        Dim SN As Double
        Dim TA As Double
        Dim TB As Double
        Dim GJ As Double
        Dim GK As Double
        Dim GJL As Double
        Dim GKL As Double
        Dim VJ As Double
        Dim VK As Double
        Dim IT As Integer
        Dim i As Integer
        Dim j As Integer
        Dim k As Integer
        Dim l As Integer

        ' 初期化
        DL = 0.000001#

        For i = 1 To nAtom
            For j = 1 To nAtom
                V(i, j) = 0.0#
            Next j
        Next i

        For i = 1 To nAtom
            V(i, i) = 1.0#
        Next i

        D = 1.0#

        Do While D > DL

            D = Math.Abs(G(2, 1))

            For i = 2 To nAtom
                For j = 1 To i - 1
                    If Math.Abs(G(i, j)) > D Then D = Math.Abs(G(i, j))
                Next j
            Next i

            SM = 0.1# * D
            IT = IT + 1

            For j = 2 To nAtom

                For k = 1 To j - 1

                    If Math.Abs(G(j, k)) > SM Then
                        GJ = G(j, j)
                        GK = G(k, k)

                        TB = (GJ - GK) / (2 * G(k, j))

                        If TB <= 0 Then TA = TB - Math.Sqrt(1 + TB * TB) Else TA = TB + Math.Sqrt(1 + TB * TB)
                        CS = 1 / Math.Sqrt(1 + TA * TA)
                        SN = CS * TA

                        For l = 1 To nAtom

                            If l = k Then
                                G(k, k) = GK * CS * CS + G(j, k) * 2 * SN * CS + GJ * SN * SN
                                G(j, j) = GK + GJ - G(k, k)
                            Else
                                If l <> j Then
                                    GKL = G(k, l)
                                    GJL = G(j, l)
                                    G(k, l) = GKL * CS + GJL * SN
                                    G(j, l) = GJL * CS - GKL * SN
                                    G(l, k) = G(k, l)
                                    G(l, j) = G(j, l)
                                End If
                            End If

                        Next l

                        G(j, k) = G(j, k) * (CS * CS - SN * SN) + (GJ - GK) * SN * CS
                        G(k, j) = G(j, k)

                        For i = 1 To nAtom
                            VJ = V(i, j)
                            VK = V(i, k)
                            V(i, j) = VJ * CS - VK * SN
                            V(i, k) = VJ * SN + VK * CS
                        Next i

                    End If

                Next k

            Next j

        Loop

    End Sub
    Sub INCRE()

        ' 固有ベクトルを固有値の順に並べかえる
        Dim i As Integer
        Dim j As Integer
        Dim iMax As Integer
        Dim eMax As Double
        Dim t As Double

        For i = 1 To nAtom
            eMax = G(i, i)
            iMax = i
            For j = i To nAtom
                If G(j, j) > eMax Then
                    iMax = j
                    eMax = G(j, j)
                End If

            Next j

            If i < iMax Then
                t = G(i, i)

                G(i, i) = G(iMax, iMax)
                G(iMax, iMax) = t

                For j = 1 To nAtom
                    t = V(j, i)
                    V(j, i) = V(j, iMax)
                    V(j, iMax) = t

                Next j

            End If

        Next i

    End Sub

    Sub PMAT()

        ' 結合次数の計算

        Dim i As Integer
        Dim j As Integer
        Dim k As Integer
        Dim NOCC As Integer
        Dim Odd As Boolean
        Dim Degenerate As Boolean
        Const EPS = 0.000001#

        NOCC = (nPiElectron + 1) \ 2
        If NOCC * 2 > nPiElectron Then Odd = True

        ' HOMOと一つ上のMOのエネルギーを比較
        If G(NOCC, NOCC) - G(NOCC + 1, NOCC + 1) < EPS Then Degenerate = True

        For i = 1 To nAtom

            For j = 1 To i

                P(i, j) = 0.0#

                For k = 1 To NOCC - 1

                    P(i, j) = P(i, j) + 2.0# * V(i, k) * V(j, k)

                Next k

                If Degenerate Then
                    If Odd Then
                        P(i, j) = P(i, j) + 0.5 * (V(i, NOCC) * V(j, NOCC) + V(i, NOCC + 1) * V(j, NOCC + 1))
                    Else
                        P(i, j) = P(i, j) + V(i, NOCC) * V(j, NOCC) + V(i, NOCC + 1) * V(j, NOCC + 1)
                    End If
                Else
                    If Odd Then
                        P(i, j) = P(i, j) + V(i, NOCC) * V(j, NOCC)
                    Else
                        P(i, j) = P(i, j) + 2.0# * V(i, NOCC) * V(j, NOCC)
                    End If

                End If

            Next j

        Next i

    End Sub

    Sub POUT()

        Dim i As Integer
        Dim j As Integer
        Dim k As Integer
        Dim m As Integer
        Dim pdat As String
        Const nItem = 6

        Console.WriteLine()
        Console.WriteLine()
        Title = "化合物名: " + Title
        Console.WriteLine(Title)
        pdat = "原子数: " + Str(nAtom)
        Console.WriteLine(pdat)
        pdat = "結合数: " + Str(nBond)
        Console.WriteLine(pdat)
        pdat = "π電子数: " + Str(nPiElectron)
        Console.WriteLine(pdat)
        Console.WriteLine()
        Console.WriteLine("-- Eigenvalues and Eigenvectors --")
        Console.WriteLine()

        ' 固有値と固有ベクトルの表示

        k = 1     ' 開始列
        m = k + nItem - 1

        Do While k <= nAtom

            If m > nAtom Then m = nAtom

            pdat = Space(9)

            For i = k To m
                pdat &= i.ToString("D2") + Space(9)
            Next i

            Console.WriteLine(pdat)


            ' 固有値（エネルギー）
            pdat = Space(2)

            For i = k To m
                pdat &= Space(3) & G(i, i).ToString(" 0.####0;-0.####0")
            Next i

            Console.WriteLine(pdat)
            Console.WriteLine()

            ' 固有ベクトル（波動関数)

 
            For i = 1 To nAtom
                pdat = i.ToString("D2")

                For j = k To m
                    pdat &= Space(3) & V(i, j).ToString(" 0.####0;-0.####0")
                Next j
                Console.WriteLine(pdat)
            Next i

            k = m + 1
            m = k + nItem - 1
            Console.WriteLine()

        Loop

        Console.WriteLine()

        ' 結合次数の表示

        Console.WriteLine("-- Bond Order Matrix --")
        Console.WriteLine()

        k = 1
        m = k + nItem - 1

        Do While k <= nAtom

            If m > nAtom Then m = nAtom

            pdat = Space(9)

            For i = k To m
                pdat &= i.ToString("D2") + Space(9)
            Next i
            Console.WriteLine(pdat)

            pdat = ""

            For i = 1 To nAtom

                pdat = i.ToString("D2")

                For j = k To m
                    pdat &= Space(3) & P(i, j).ToString(" 0.####0;-0.####0")
                Next j

                Console.WriteLine(pdat)
            Next i

            k = m + 1
            m = k + nItem - 1
            Console.WriteLine()
        Loop

    End Sub

End Module
